import java.util.regex.Pattern;

import org.apache.commons.math3.util.ArithmeticUtils;

public class Poly {
	
	private static Pattern exponentPattern = Pattern.compile("\\(.*\\).*");
	//single variable polynomial of max 10 terms, 
	//optionally an exponent for the entire (linear) polynomial
	public Term[] terms = new Term[10];
	public Integer[] coeffs = new Integer[10];
	public int exp = 1;
	
	public Poly() {		
	}
	
	public Poly(String poly) {
		this();
		if(exponentPattern.matcher(poly).matches()) {
			String[] polyParts = poly.split("\\*\\*");
			exp = Integer.valueOf(polyParts[1]);
			poly = polyParts[0].replaceAll("\\(", "").replaceAll("\\)", "");
		}
		String[] curTerms = poly.replaceAll("\\-", "\\+\\-").split("\\+");
		for(String curTerm : curTerms) {
			addTerm(curTerm);
		}
	}
	
	public static Poly newPoly() {
		return new Poly();
	}
	
	public static Poly newPoly(String poly) {
		return new Poly(poly);
	}
	
	public void addTerm(String pattern) {
		if(pattern == null || pattern.isEmpty())
			return;
		if(pattern.matches(".*[a-zA-Z].*")) {
			int expIdx = pattern.indexOf("**"), coeff = 1;
			try {
				String sCoeff = pattern.split("[a-zA-Z]")[0];
				if(sCoeff.contains("+") || sCoeff.contains("-"))
					sCoeff = sCoeff.replaceAll("\\-", "\\+\\-").split("\\+")[1];
				coeff = "-".equals(sCoeff)?-1:Integer.valueOf(sCoeff);
			} catch(Exception e){/*swallow*/}
			
			int exp = 1;
			if(expIdx != -1){
				exp = Integer.valueOf(pattern.substring(pattern.indexOf("**")+2));
			}

			if (coeffs[exp]==null) coeffs[exp] = 0;
			coeffs[exp] = coeffs[exp]+coeff;
			terms[exp] = coeff!=0?new Term(new Sym("x")):null;
		} else {
			if (coeffs[0]==null) coeffs[0] = 0;
			String sCoeff = pattern.split("[a-zA-Z]")[0];
			if(sCoeff.contains("+") || sCoeff.contains("-"))
				sCoeff = sCoeff.replaceAll("\\-", "\\+\\-").split("\\+")[1];
			coeffs[0] = coeffs[0]+("-".equals(sCoeff)?-1:Integer.valueOf(sCoeff));
			terms[0] = new Term();
		}
	}
	
	public void removeTerm(String pattern) {
		if(pattern == null || pattern.isEmpty() || !pattern.matches(".*[a-zA-Z].*"))
			return;
		int expIdx = pattern.indexOf("**");
		int exp = 1;
		if(expIdx != -1) {
			exp = Integer.valueOf(pattern.substring(pattern.indexOf("**")+2));
		}		
		terms[exp] = null;
		coeffs[exp] = null;
	}
	
	public int findCoeff(String pattern) {
		Poly tempPoly = new Poly(pattern);
		for(int i=0; i<tempPoly.terms.length; i++) {
			if(tempPoly.terms[i] != null) {
				if (terms[i].hasSym(tempPoly.terms[i].syms.get(0)))
					return coeffs[i];
			}
		}
		return 0;
	}
	
	public int binomialCoefficient(int n, int k){
		return (int)ArithmeticUtils.binomialCoefficient(n, k);
	}
	
	public int pow(int k, int e) {
		return ArithmeticUtils.pow(k, e);
	}
	
	public String toString() {
		StringBuilder polyString = new StringBuilder();
		for(int i=coeffs.length-1; i>=0; i--) {
			if(coeffs[i]==null) continue;
			polyString.append(coeffs[i]>0&&polyString.length()>0?"+":"");
			polyString.append(coeffs[i]!=1||terms[i].syms.size()==0?coeffs[i]:"");
			polyString.append(terms[i].syms.size()>0?terms[i].syms.get(0).symbol:"");
			polyString.append(i>1?"**"+i:"");
		}
		if(exp != 1) {
			StringBuilder fullPolyString = new StringBuilder("(").append(polyString).append(")").append("**").append(exp);
			return fullPolyString.toString();
		}
		return polyString.toString();
	}
	
	public static void main(String[] args) {
		Poly p = new Poly("");
		System.out.println(p);
		p = new Poly("x**3-3x**2+5x-1");
		System.out.println(p);
		p.addTerm("+4x");
		System.out.println(p);
		System.out.println(p.findCoeff("x**2"));
		p.removeTerm("x**2");
		System.out.println(p);
	}

}
